import urllib.parse as urlparse
from typing import Generator, Literal, override

from ab_core.auth_flow.oauth2.schema.auth_code_stage import (
    AuthCodeStageInfo,
    AuthCodeStageInfoBeginLogin,
    AuthCodeStageInfoDone,
)
from ab_core.auth_flow.oauth2.schema.flow_type import (
    OAuth2FlowType,
)
from ab_core.impersonation.impersonator import Impersonator

from .base import OAuth2FlowBase


class ImpersonationOAuth2Flow(OAuth2FlowBase):
    """Automate browser login to capture auth code via OIDC with PKCE."""

    type: Literal[OAuth2FlowType.IMPERSONATION] = OAuth2FlowType.IMPERSONATION
    impersonator: Impersonator

    @override
    def get_code(
        self, authorize_url: str
    ) -> Generator[AuthCodeStageInfo, None, AuthCodeStageInfoDone]:
        with self.impersonator.init_context(authorize_url) as context:
            # prepare the user interaction
            interaction = self.impersonator.init_interaction(context)
            if interaction:
                yield AuthCodeStageInfoBeginLogin(
                    ws_url=interaction.ws_url,
                    gui_url=interaction.gui_url,
                )

            # intercept the response during user interaction
            with self.impersonator.intercept_response(
                context,
                cond=lambda r: r.url.startswith(str(self.idp_prefix)) and r.status == 302,
                timeout=self.timeout,
            ) as resp:
                loc = resp.headers.get("location")
                if not loc:
                    raise RuntimeError(
                        "Unable to extract Auth Code: No location found in response headers."
                    )
                auth_code = urlparse.parse_qs(urlparse.urlparse(loc).query).get("code", [None])[0]

            auth_code_done_stage = AuthCodeStageInfoDone(auth_code=auth_code)
            yield auth_code_done_stage
            return auth_code_done_stage



from abc import ABC
from enum import StrEnum
from typing import Annotated, Literal, Optional, Tuple

from pydantic import BaseModel, Discriminator


class AuthCodeStage(StrEnum):
    BEGIN_LOGIN = "BEGIN_LOGIN"  # redirect the user to begin the login
    DONE = "DONE"  # the auth code has been retrieved


class AuthCodeStageInfoBase(BaseModel, ABC): ...


class AuthCodeStageInfoBeginLogin(AuthCodeStageInfoBase):
    stage: Literal[AuthCodeStage.BEGIN_LOGIN] = AuthCodeStage.BEGIN_LOGIN

    ws_url: str
    gui_url: Optional[str] = None


class AuthCodeStageInfoDone(AuthCodeStageInfoBase):
    event: Literal[AuthCodeStage.DONE] = AuthCodeStage.DONE

    auth_code: str


AuthCodeStageInfo = Annotated[
    Tuple[AuthCodeStageInfoBeginLogin, AuthCodeStageInfoDone],
    Discriminator("stage"),
]



from abc import ABC, abstractmethod
from typing import Generator

from pydantic import AnyHttpUrl, BaseModel

from ab_core.auth_flow.oauth2.schema.auth_code_stage import (
    AuthCodeStageInfo,
    AuthCodeStageInfoDone,
)


class OAuth2FlowBase(BaseModel, ABC):
    """Automate browser login to capture auth code via OIDC with PKCE."""

    idp_prefix: AnyHttpUrl
    timeout: int

    @abstractmethod
    def get_code(
        self,
        authorize_url: str,
    ) -> Generator[AuthCodeStageInfo, None, AuthCodeStageInfoDone]: ...



from typing import Generator, Literal, override

from ab_core.auth_flow.oauth2.schema.auth_code_stage import (
    AuthCodeStageInfo,
    AuthCodeStageInfoDone,
)
from ab_core.auth_flow.oauth2.schema.flow_type import (
    OAuth2FlowType,
)

from .base import OAuth2FlowBase


class TemplateOAuth2Flow(OAuth2FlowBase):
    """Automate browser login to capture auth code via OIDC with PKCE."""

    type: Literal[OAuth2FlowType.TEMPLATE] = OAuth2FlowType.TEMPLATE

    @override
    def get_code(
        self,
        authorize_url: str,
    ) -> Generator[AuthCodeStageInfo, None, AuthCodeStageInfoDone]:
        raise NotImplementedError()



from typing import Annotated, Union

from pydantic import Discriminator

from .impersonation import ImpersonationOAuth2Flow
from .template import TemplateOAuth2Flow

OAuth2Flow = Annotated[
    Union[ImpersonationOAuth2Flow, TemplateOAuth2Flow],
    Discriminator("type"),
]
